@@ -36,11 +36,6 @@
 
 #include <libgnomeui/gnome-window-icon.h>
 
-#ifdef HAVE_RANDR
-#include <gdk/gdkx.h>
-#include <X11/extensions/Xrandr.h>
-#endif
-
 #include "manager.h"
 #include "ice.h"
 #include "save.h"
@@ -49,6 +44,8 @@
 #include "util.h"
 #include "gsm-sound.h"
 #include "gsm-gsd.h"
+#include "gsm-keyring.h"
+#include "gsm-xrandr.h"
 #include "gsm-at-startup.h"
 
 /* Flag indicating autosave - user won't be prompted on logout to 
@@ -288,197 +285,31 @@ gnome_login_check (void)
     }
 }
 
-#ifdef HAVE_RANDR
-static gboolean
-get_resolution (GConfClient *client, int screen, char *keys[], int *width, int *height)
+static void
+gsm_shutdown_gconfd (void)
 {
-  int i;
-  char *key;
-  char *val;
-  int w, h;
-
-  val = NULL;
-  for (i = 0; keys[i] != NULL; i++)
-    {
-      key = g_strdup_printf ("%s/%d/resolution", keys[i], screen);
-      val = gconf_client_get_string (client, key, NULL);
-      g_free (key);
-
-      if (val != NULL)
-	break;
-    }
-  
-  if (val == NULL)
-    return FALSE;
-
-  if (sscanf (val, "%dx%d", &w, &h) != 2)
-    return FALSE;
-
-  *width = w;
-  *height = h;
-  
-  return TRUE;
-}
-
-static int
-get_rate (GConfClient *client, int screen, char *keys[])
-{
-  int i;
-  char *key;
-  int val;
   GError *error;
+  char   *command;
+  int     status;
 
-  for (i = 0; keys[i] != NULL; i++)
+  command = g_strjoin (" ", GCONFTOOL_CMD, "--shutdown", NULL);
+
+  status = 0;
+  error  = NULL;
+  if (!g_spawn_command_line_sync (command, NULL, NULL, &status, &error))
     {
-      key = g_strdup_printf ("%s/%d/rate", keys[i], screen);
-      error = NULL;
-      val = gconf_client_get_int (client, key, &error);
-      g_free (key);
-
-      if (error == NULL)
-	return val;
-
+      g_warning ("Failed to execute '%s' on logout: %s\n",
+		 command, error->message);
       g_error_free (error);
     }
-  
-  return 0;
-}
 
-static int
-find_closest_size (XRRScreenSize *sizes, int nsizes, int width, int height)
-{
-  int closest;
-  int closest_width, closest_height;
-  int i;
-
-  closest = 0;
-  closest_width = sizes[0].width;
-  closest_height = sizes[0].height;
-  for (i = 1; i < nsizes; i++)
+  if (status)
     {
-      if (ABS (sizes[i].width - width) < ABS (closest_width - width) ||
-	  (sizes[i].width == closest_width &&
-	   ABS (sizes[i].height - height) < ABS (closest_height - height)))
-	{
-	  closest = i;
-	  closest_width = sizes[i].width;
-	  closest_height = sizes[i].height;
-	}
-    }
-  
-  return closest;
-}
-
-#endif
-
-static void
-set_display_properties (void)
-{
-#ifdef HAVE_RANDR
-  GdkDisplay *display;
-  Display *xdisplay;
-  int major, minor;
-  int event_base, error_base;
-  int n_screens;
-  GdkScreen *screen;
-  GdkWindow *root_window;
-  int width, height, rate;
-  GConfClient *client;
-#ifdef HOST_NAME_MAX
-  char hostname[HOST_NAME_MAX + 1];
-#else
-  char hostname[256];
-#endif
-  char *specific_path;
-  char *keys[3];
-  int i;
-
-  display = gdk_display_get_default ();
-  xdisplay = gdk_x11_display_get_xdisplay (display);
-
-  /* Check if XRandR is supported on the display */
-  if (!XRRQueryExtension (xdisplay, &event_base, &error_base) ||
-      XRRQueryVersion (xdisplay, &major, &minor) == 0)
-    return;
-  
-  if (major != 1 || minor < 1)
-    {
-      g_message ("Display has unsupported version of XRandR (%d.%d), not setting resolution.", major, minor);
-      return;
+      g_warning ("Running '%s' at logout returned an exit status of '%d'",
+		 command, status);
     }
 
-  client = gconf_client_get_default ();
-  
-  keys[0] = "/desktop/gnome/screen/default";
-  specific_path = NULL;
-  if (gethostname (hostname, sizeof (hostname)) != 0)
-    specific_path = g_strconcat ("/desktop/gnome/screen/", hostname,  NULL);
-  keys[1] = specific_path;
-  keys[2] = NULL;
-  
-  n_screens = gdk_display_get_n_screens (display);
-  for (i = 0; i < n_screens; i++)
-    {
-      screen = gdk_display_get_screen (display, i);
-      root_window = gdk_screen_get_root_window (screen);
-
-      if (get_resolution (client, i, keys, &width, &height))
-	{
-	  XRRScreenSize *sizes;
-	  int nsizes, j;
-	  int closest;
-	  short *rates;
-	  int nrates;
-	  int status;
-	  int current_size;
-	  short current_rate;
-	  XRRScreenConfiguration *config;
-	  Rotation current_rotation;
-
-	  config = XRRGetScreenInfo (xdisplay, gdk_x11_drawable_get_xid (GDK_DRAWABLE (root_window)));
-
-	  rate = get_rate (client, i, keys);
-	  
-	  sizes = XRRConfigSizes (config, &nsizes);
-	  closest = find_closest_size (sizes, nsizes, width, height);
-	  
-	  rates = XRRConfigRates (config, closest, &nrates);
-	  for (j = 0; j < nrates; j++)
-	    {
-	      if (rates[j] == rate)
-		break;
-	    }
-	  /* Rate not supported, let X pick */
-	  if (j == nrates)
-	    rate = 0;
-
-	  current_size = XRRConfigCurrentConfiguration (config, &current_rotation);
-	  current_rate = XRRConfigCurrentRate (config);
-
-	  if (closest != current_size ||
-	      rate != current_rate)
-	    {
-	      status = XRRSetScreenConfigAndRate (xdisplay, 
-						  config,
-						  gdk_x11_drawable_get_xid (GDK_DRAWABLE (root_window)),
-						  closest,
-						  current_rotation,
-						  rate,
-						  GDK_CURRENT_TIME);
-	    }
-	}
-    }
-	  
-  g_object_unref (client);
-  
-  g_free (specific_path);
-
-  /* We need to make sure we process the screen resize event. */
-  gdk_display_sync (display);
-  while (gtk_events_pending ())
-    gtk_main_iteration();
-  
-#endif
+  g_free (command);
 }
 
 int
@@ -511,8 +342,15 @@ main (int argc, char *argv[])
   gtk_init (&argc, &argv);
 
   /* We need to do this as early as possible */
-  set_display_properties ();
+  gsm_set_display_properties ();
   
+  /* Make sure the splash screen has the SM_CLIENT_ID
+   * property set so smproxy doesn't try and session
+   * manage us.
+   * See http://bugzilla.gnome.org/show_bug.cgi?id=118063
+   */
+  gdk_set_sm_client_id ("gnome-session-dummy-id");
+
   gnome_login_check ();
 
   err = NULL;
@@ -558,6 +396,9 @@ main (int argc, char *argv[])
 
   ignore (SIGPIPE);
 
+  /* Need DISPLAY set */
+  gsm_keyring_daemon_start ();
+  
   /* Read in config options */  
   gconf_client = gconf_client_get_default ();
 
@@ -625,6 +466,10 @@ main (int argc, char *argv[])
 
   gsm_sound_logout ();
 
+  gsm_keyring_daemon_stop ();
+  
+  gsm_shutdown_gconfd ();
+
   clean_ice ();
 
   /* If a logout command was set, the following will not return */
